/**
   TProcess
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;


// Imported SAX classes

import org.xml.sax.SAXException;
import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.Locator ;


/**
  * this class allow to define a stack for content handler event.
  * This class is manage by the MultipleMixer class
  
  * @author laurent.Tardif 
  * @ since jdk 1.3 && SAX 2.0
*/
public final class LoopStackMixer
    implements ContentHandler, ContentHandler_Commands {

  /** the stack of content handler info */
  ContentHandlerInfoStack iCHSI = new ContentHandlerInfoStack();
  
  /** true if the content handler is the current one*/
  boolean isCurrent = false ;
  
  /** true if the mixer is finished */
  boolean isDone = false ;
  
  /** the manager */
  OneMixerHandler iOwner = null;

  
  /** the content handler */
  ContentHandler iContentHandler = null;
 
  /** the constructor */
  public LoopStackMixer (OneMixerHandler pOwner ) {
    iOwner = pOwner ;
  }
   
  /** Set the content handler for this element */
  public final void setContentHandler(ContentHandler pContentHandler ) {
    iContentHandler = pContentHandler ;
  }

  /** get the content handler for this element */
  public final ContentHandler getContentHandler( ) {
    return iContentHandler;
  } 
  
  public final ContentHandlerInfoStack getContentHandlerInfoStack() {
    return iCHSI ;
  }
  

  /** The startElement command */
  public final void startElement(String ns,String localName,String name, Attributes atts) throws SAXException {
    if (isCurrent) {
      getContentHandler().startElement(ns,localName,name,atts);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_ELEMENT,ns,localName,name,atts));      
      //System.out.println("Mixer . startElement ("+ns+", "+localName+", "+name+","+atts+")");
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_ELEMENT,ns,localName,name,atts));
      //System.out.println("Mixer . startElement delayed("+ns+", "+localName+", "+name+","+atts+")");
    }
  }

  /** The endElement command */
  public final void endElement(String ns,String localName,String name) throws SAXException {
    if (isCurrent) {
      getContentHandler().endElement(ns,localName,name);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_ELEMENT,ns,localName,name));
      //System.out.println("Mixer . endElement ("+ns+", "+localName+", "+name+")");
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_ELEMENT,ns,localName,name));
      //System.out.println("Mixer . endElement delayed("+ns+", "+localName+", "+name+")");
    }
  }

  /** The endElement command */
  public final void startDocument() throws SAXException {
    if (isCurrent) {
      //System.out.println("Mixer . startDocument ");
     getContentHandler().startDocument();
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_DOCUMENT));
    } else {
      //System.out.println("Mixer . startDocument delayed");
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_DOCUMENT));
    }
  }

  /** The endElement command */
  public final void setDocumentLocator(Locator locator) {
      if (isCurrent) {
       getContentHandler().setDocumentLocator(locator);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.SET_DOCUMENT_LOCATOR,locator));
      //System.out.println("Mixer . setDocumentLocator ");
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.SET_DOCUMENT_LOCATOR,locator));
      //System.out.println("Mixer . setDocumentLocator delayed");
    }
  }


  /** The endElement command */
  public synchronized final void endDocument() throws SAXException {
    if (isCurrent) {
      //System.out.println("Mixer . endDocument ");
      getContentHandler().endDocument();
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_DOCUMENT));
      iOwner.IsFinished();
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_DOCUMENT));
      iOwner.IsFinishedDuringStop();
      //System.out.println("Mixer . endDocument delayed"+ iCHSI);
    }
  }

  /** The skippedEntity command */
  public final void skippedEntity(java.lang.String name) throws SAXException{
    if (isCurrent) {
      //System.out.println("Mixer . skippedEntity " );
     getContentHandler().skippedEntity(name);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.SKIPPED_ENTITY,name));
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.SKIPPED_ENTITY,name));
      //System.out.println("Mixer . skippedEntity delayed");
    }
  }


  /** The processingInstruction command */
  public final void processingInstruction(java.lang.String target, java.lang.String data) throws SAXException{
    if (isCurrent) {
      //System.out.println("Mixer . processingInstruction " );
     getContentHandler().processingInstruction(target,data);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.PROCESSING_INSTRUCTION,target,data));
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.PROCESSING_INSTRUCTION,target,data));
      //System.out.println("Mixer . processingInstruction delayed");
    }
  }


  /** The ignorableWhitespace command */
  public final void ignorableWhitespace(char[] ch, int start, int length) throws SAXException{
    if (isCurrent) {
     getContentHandler().ignorableWhitespace(ch,start,length);
      //System.out.println("Mixer . ignorableWhitespace " );
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.IGNORABLE_WHITESPACE,ch,start,length));
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.IGNORABLE_WHITESPACE,ch,start,length));
      //System.out.println("Mixer . ignorableWhitespace delayed");
    }
  }

  /** The characters command */
  public final void characters(char[] ch, int start, int length) throws SAXException{
    if (isCurrent) {
     getContentHandler().characters(ch,start,length);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.CHARACTERS,ch,start,length));
      /*
      String toto = "";
      for (int i=start; i<length+start; i++) toto+=ch[i];
      System.out.println("Mixer . characters ; " + toto );
      */
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.CHARACTERS,ch,start,length));
      //System.out.println("Mixer . characters delayed");
    }
  }

  /** The startPrefixMapping command */
  public final void startPrefixMapping(java.lang.String prefix, java.lang.String uri) throws SAXException{
    if (isCurrent) {
     getContentHandler().startPrefixMapping(prefix,uri);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_PREFIX_MAPPING,prefix,uri));
      //System.out.println("Mixer . startPrefixMapping " );
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.START_PREFIX_MAPPING,prefix,uri));
      //System.out.println("Mixer . startPrefixMapping delayed");
    }
  }

  /** The endPrefixMapping command */
  public final void endPrefixMapping(java.lang.String prefix) throws SAXException{
    if (isCurrent) {
     getContentHandler().endPrefixMapping(prefix);
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_PREFIX_MAPPING,prefix));
      //System.out.println("Mixer . endPrefixMapping " );
    } else {
      iCHSI.add(new ContentHandlerInfo(ContentHandlerInfo.END_PREFIX_MAPPING,prefix));
      //System.out.println("Mixer . endPrefixMapping delayed");
    }
  }

  /** set the curent attribut */
  public final void setCurrent (boolean pCurrent) {
    isCurrent = pCurrent ;
  }


  /** Set the done attribute */
  public final void setDone (boolean pDone) {
    isDone = pDone ;
  }

  /** Test if the mixer is done */
  public final boolean isDone() {
    return isDone ;
  }

  /** test if the mixer is the current mixer */
  public final boolean isCurrent() {
    return isCurrent ;
  }

  /** Compute the stack of contentHandlerInfo */
  public final void ComputeStack() throws SAXException{
    ContentHandlerInfo current = null ;
    int size = iCHSI.size() ;
    for (int i=0; i< size; i++) {
      current= iCHSI.pop();
       //System.out.println("Current execution :"+i+" ="+current );
       ExecuteContentHandlerInfo(current);
    }
  }
  
  /** Execute one ContentHandlerInfo in the stack*/
  protected final void  ExecuteContentHandlerInfo (ContentHandlerInfo current) throws SAXException{
    switch(current.getCommand() ) {
      case START_ELEMENT :
       getContentHandler().startElement(current.getStringParameter(0),
                            current.getStringParameter(1),
                            current.getStringParameter(2),
                            current.getAttributes());
        //System.out.println("Mixer . startElement delayed (execution of stack) ("+current.getStringParameter(2)+", "+current.getAttributes()+")");                            
        break;
      case END_ELEMENT :
       //System.out.println("Mixer . endElement delayed (execution of stack) ("+current.getStringParameter(2)+")");
       getContentHandler().endElement(current.getStringParameter(0),
                          current.getStringParameter(1),
                          current.getStringParameter(2));
        break ;
      case START_DOCUMENT :
       getContentHandler().startDocument();
        break ;
      case END_DOCUMENT :
       getContentHandler().endDocument();
        System.out.println("Mixer . endDocument delayed (execution of stack)");
        iOwner.IsFinished();
        break ;
      case SET_DOCUMENT_LOCATOR :
       getContentHandler().setDocumentLocator(current.getLocatorParameter());
        break ;
      case SKIPPED_ENTITY :
       getContentHandler().skippedEntity(current.getStringParameter(0));
        break ;
      case IGNORABLE_WHITESPACE :
       getContentHandler().ignorableWhitespace(current.getCharParameter(),
                                   current.getIntParameter(0),
                                   current.getIntParameter(1));
        break ;
      case PROCESSING_INSTRUCTION :
       getContentHandler().processingInstruction(current.getStringParameter(0),
                                     current.getStringParameter(1));
        break ;
      case CHARACTERS :
       getContentHandler().characters(current.getCharParameter(),
                          current.getIntParameter(0),
                          current.getIntParameter(1));
        break ;
      case START_PREFIX_MAPPING :
       getContentHandler().startPrefixMapping(current.getStringParameter(0),
                                  current.getStringParameter(1));
        break ;
      case END_PREFIX_MAPPING :
       getContentHandler().endPrefixMapping(current.getStringParameter(0));
        break ;
      default :
        System.out.println("[LoopStackMixer]  ExecuteContentHandlerInfo : commandes inconnues");
    } //end switch
         
  } //end ExecuteContentHandlerInfo
  
} //end class
