/**
   TProcess
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;


// Imported SAX classes

import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.Locator ;
import org.xml.sax.SAXException;

/** 
  * this class define the dispatcher process.
  * dispatch one data flow to different process.
  *
  * @ author laurent tardif 
  * @date 10 02 2001
  * @since jdk 1.2 SAX 2.0
  */

public final class TDispatcher 
  extends TProcessBasic 
  implements ContentHandler {

  /** a list of content handler to dispatch information */
  ContentHandler [] iListOfContent = null ;

  /** the constructor */
  public TDispatcher (String[] pIn, String[] pOut) {
    super(pIn,pOut);
    //System.out.println("{ TProcess]Creation d'un dispatcher");
    iListOfContent = new ContentHandler[iNbOut];
    for (int i=0; i<iNbOut; i++) {
      iListOut [i] = new XML_Port(pOut[i],this,this,i,XML_Port.OUT);
    } //end for
    iListIn[0]= new XML_Port(pIn[0],this,(ContentHandler)this,0,XML_Port.IN);  
  } //end constructor


 
  /** set a content handler */
  public final void setContentHandler(int i, ContentHandler pContentHandler) {
     iListOfContent[i] = pContentHandler ;
  }
  
  // a set of method to implement content handler interface
  
  /** the start element method of content handler interface */
  public final void startElement(String ns,String localName,String name, Attributes atts) throws SAXException {
    
   /* System.out.println("[TDispatcher("+getName()+"] startElement =>" +ns +","+localName+","+name+","+atts);
   if (atts != null) {
      int len = atts.getLength();
      for (int i = 0; i < len; i++) {
        System.out.print("AttName="+atts.getQName(i));
        System.out.println("Value="+atts.getValue(i));        
      } //end for
    } 
    */
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].startElement( ns, localName, name,  atts);
    }
  }

  /** the endElement method of content handler interface */
  public final void endElement(String ns,String localName,String name) throws SAXException {
    //System.out.println("[TDispatcher] endElement=>" +ns +","+localName+","+name);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].endElement( ns, localName, name);
    }
  }

  /** the setDocumentLocator method of content handler interface */
  public final void setDocumentLocator(Locator locator) {
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].setDocumentLocator( locator);
    }
  }

  /** the startDocument method of content handler interface */
  public final void startDocument() throws SAXException {
    //System.out.println("[TDispatcher] startDocument=>");    
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].startDocument();
    }
  }

  /** the endDocument method of content handler interface */
  public final void endDocument() throws SAXException {
    //System.out.println("[TDispatcher] endDocument=>");
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].endDocument();
    }
  }

  /** the skippedEntity method of content handler interface */
  public final void skippedEntity(java.lang.String name) throws SAXException {
    //System.out.println("[TDispatcher] skippedEntity=>" +name);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].skippedEntity( name);
    }
  }

  /** the processingInstruction method of content handler interface */
  public final void processingInstruction(java.lang.String target, java.lang.String data) throws SAXException {
    //System.out.println("[TDispatcher] processingInstruction=>" +target+","+data);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].processingInstruction(target, data);
    }
  }

  /** the ignorableWhitespace method of content handler interface */
  public final void ignorableWhitespace(char[] ch, int start, int length) throws SAXException{
   /*String titi="";
    for (int i=start; i<start+length; i++){
      titi+=ch[i];
    }
    System.out.println("ignorableWhitespace" + titi);
    */
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].ignorableWhitespace( ch,  start,  length);
    }
  }

  /** the characters method of content handler interface */
  public final void characters(char[] ch, int start, int length) throws SAXException{
    /*String titi="";
    for (int i=start; i<start+length; i++){
      titi+=ch[i];
    }
    System.out.println("characters" + titi);
    */
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].characters( ch,  start,  length);
    }
  }

  /** the startPrefixMapping method of content handler interface */
  public final void startPrefixMapping(java.lang.String prefix, java.lang.String uri) throws SAXException{
    //System.out.println("[TDispatcher] startPrefixMapping=>" +prefix+","+uri);
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].startPrefixMapping( prefix,uri);
    }
  }

  /** the endPrefixMapping method of content handler interface */
  public final void endPrefixMapping(java.lang.String prefix) throws SAXException{
    //System.out.println("[TDispatcher] endPrefixMapping=>" +prefix);  
    for (int i=0; i<iNbOut; i++) {
      iListOfContent[i].endPrefixMapping( prefix);
    }
  }

  /**
     The copy of this element. The method was call by the copy method
     of the _Process_impl class 
  */ 
   
  public final TProcess internalCopy() {
    String[] vPortName = new String [getOut().length];
    XML_Port[] out = getOut();
    for (int i =0; i<vPortName.length; i++) {
      vPortName[i]=out[i].getCompositeName();
    }

    String[] vPortNameIn = new String [getIn().length];
    XML_Port[] in= getIn();
    for (int i =0; i<vPortNameIn.length; i++) {
      vPortNameIn[i]=in[i].getCompositeName();
    }
    TDispatcher vAns = new TDispatcher(vPortNameIn,vPortName);
    return vAns ;
  }
  

} //end class
